PREF = {
    'clear_vowel': (True, False),
    'clear_vowel_onset': ('not_true_cluster', 'all'),
    'clear_vowel_tone_mark': (True, False),
    'split_true_cluster': (True, False),
    'split_false_cluster': (True, False),
    'split_leading_con': (True, False),
    'obvious_low_singles': (True, False),
    'obvious_h_low_single': (True, False),
    'onset_style': ('plain', 'phinthu', 'yaamakkaan', 'kaaran'),
    'onset_style_apply': ('not_h', 'h'),
    'silent_before_style':
        ('plain', 'phinthu', 'yaamakkaan', 'kaaran', 'hide'),
    'coda_style': ('plain', 'phinthu', 'yaamakkaan', 'kaaran'),
    'silent_after_style':
        ('plain', 'phinthu', 'yaamakkaan', 'kaaran', 'hide'),
    'vowel_no_coda': ('pair', 'silent_after'),
    'vowel_coda_form': (),
    'vowel_length': ('input', 'short', 'long'),
    'vowel_pair_form': (),
    'low_single_h_thoo': (True, False)
}

_DEFAULT_PREF = {
    'clear_vowel': True,
    'clear_vowel_onset': 'not_true_cluster',
    'clear_vowel_tone_mark': False,
    'split_true_cluster': False,
    'split_false_cluster': False,
    'split_leading_con': False,
    'obvious_low_singles': True,
    'obvious_h_low_single': True,
    'onset_style': 'plain',
    'onset_style_apply': 'not_h',
    'silent_before_style': 'kaaran',
    'coda_style': 'plain',
    'silent_after_style': 'kaaran',
    'vowel_no_coda': 'pair',
    'vowel_coda_form': {},
    'vowel_length': 'input',
    'vowel_pair_form': {},
    'low_single_h_thoo': False
}